# -*- coding: utf-8 -*-
"""
Created on Sun Nov 29 12:19:39 2020

@author: Lucia
"""

import pandas as pd
import numpy as np

import hdbscan

from sklearn.metrics import pairwise_distances

from sklearn.metrics import silhouette_score
from sklearn.metrics import calinski_harabasz_score
from sklearn.metrics import davies_bouldin_score


#------------------------------------------------------------------------------
def do_clustering(dist_matrix):

    clusterer = hdbscan.HDBSCAN(metric = 'precomputed')
    clusterer.fit(dist_matrix)
    
    labels = clusterer.labels_

    if len(np.unique(labels)) < 2:
        n_clusters = 0 if -1 in labels else 1
        n_noise = len(labels) if -1 in labels else 0
        silh_coef = -1
        cal_hara_coef = 1
        db_coef = 0

    else:
        n_clusters = len(set(labels)) - (1 if -1 in labels else 0)
        n_noise = list(labels).count(-1)
        silh_coef = silhouette_score(dist_matrix,
                                      labels,
                                      metric = 'precomputed')
        cal_hara_coef = calinski_harabasz_score(dist_matrix,
                                                labels)
        db_coef = davies_bouldin_score(dist_matrix,
                                        labels)
                
    return(n_clusters, n_noise, silh_coef, cal_hara_coef, db_coef)
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def do_optimization(dist_matrix, ind):
    
    # Listas que almacenan la combinación de parámetros y los resultados
    indss = list() # Índices de similitud
    clusters = list() # Resultados del número de clusters
    noise = list() # Resultados del número de outliers
    silhouettes = list() # Resultados del coeficiente de silhouette
    calinskis = list() # Resultados del score de calinski-harabasz
    dbs = list() # Resultados del score de davies-bouldin  

    n_clusters, n_noise, silh_coef, cal_hara_coef, db_coef = do_clustering(dist_matrix)
    
    # Guardamos los parámetros y los resultados
    indss.append(ind)
    clusters.append(n_clusters)
    noise.append(n_noise)
    silhouettes.append(silh_coef)
    calinskis.append(cal_hara_coef)
    dbs.append(db_coef)
    
    results = pd.DataFrame(list(zip(indss,
                                    clusters,
                                    noise,
                                    silhouettes,
                                    calinskis,
                                    dbs)),
                            columns = ['Similarity',
                                       'Cluster',
                                       'Noise',
                                       'Silhouette',
                                       'Calinski-Harabasz',
                                       'Davies-Bouldin'])
    
    return(results)
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def simluations(n_simulations, original, index, epsilon, ms, feature, vector, dataset):
    
    original_np = original.to_numpy()
    
    clusters = list()
    noise = list()
    silhouettes = list()
    calinskis = list() 
    dbs = list() 
    
    for n in range(n_simulations):
        
        shuffled = np.copy(original_np)
    
        # Shuffling elements in each column of the matrix
        for i in range(shuffled.shape[1]):
            np.random.shuffle(shuffled[:,i])
    
        distances = pairwise_distances(shuffled,
                                       metric = index)
    
   
        n_clusters, n_noise, silh_coef, cal_hara_coef, db_coef = do_clustering(distances)
        
        clusters.append(n_clusters)
        noise.append(n_noise)
        silhouettes.append(silh_coef)
        calinskis.append(cal_hara_coef)
        dbs.append(db_coef)
        
    
    results = pd.DataFrame(list(zip(clusters, noise, silhouettes, calinskis, dbs)),
                          columns = ['Clusters', 'Noise', 'Silhouette', 'Calinski-Harabasz', 'Davies-Bouldin'])
    
    
    results.to_csv("results/simulations/hdbscan/" + dataset + '/' + feature + "s_" + vector + '_' + index + "_ms" + str(ms) + "_eps" + str(epsilon) + ".csv", index=False)
#------------------------------------------------------------------------------