# -*- coding: utf-8 -*-
"""
Created on Sun Nov 29 12:26:56 2020

@author: Lucia
"""

import matplotlib.pyplot as plt

import numpy as np
import pandas as pd

from scipy.cluster.hierarchy import dendrogram, linkage
from scipy.spatial.distance import squareform

from sklearn.cluster import AgglomerativeClustering 

from sklearn.metrics import silhouette_score
from sklearn.metrics import calinski_harabasz_score
from sklearn.metrics import davies_bouldin_score




#------------------------------------------------------------------------------
def get_linkage(dist_matrix, link_method, ind):
    
    if ind == 'euclidean':
        link_matrix = linkage(np.triu(dist_matrix), link_method)
    else:
        link_matrix = linkage(squareform(dist_matrix), link_method)
    
    return(link_matrix)
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def plt_one_dendrogram(dist_matrix, link_method, p, ind):
    
    dendrogram(get_linkage(dist_matrix, link_method, ind), p=p, truncate_mode='lastp')
    plt.show()
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def plt_sub_dendrogram(link_matrix_list, vector, features, indexes, dataset):
       
    if vector == 'bool':
        fig, axes = plt.subplots(len(features), 
                                 len(indexes), 
                                 figsize=(11,8), 
                                 sharex=False,
                                 sharey=False)
        v = 'binary'
     
    else:
         fig, axes = plt.subplots(len(features), 
                                 len(indexes), 
                                 figsize=(11,4), 
                                 sharey=False)
         v = 'numeric'
         
            
    cols = ['{}'.format(col) for col in indexes]
    rows = ['{}s'.format(row) for row in features]
    
    for ax, col in zip(axes[0], cols):
        ax.set_title(col)
    
    for ax, row in zip(axes[:,0], rows):
        ax.set_ylabel(row, rotation=90, size='large')
    
        
    fig.tight_layout()
    
    
    fig.text(0.5, 
             1, 
             'Dendrograms in the case of the ' + dataset.lower() + ' dataset and ' + v + ' vectors', 
             ha='center',
             fontweight = "bold", fontsize=14)
    
    for n in range(len(features)):
        for p in range(len(indexes)):
            dendrogram(link_matrix_list[n][p], 
                       ax=axes[n,p],
                       p=40, 
                       leaf_rotation=60.,
                       leaf_font_size=6.,
                       truncate_mode='lastp',
                       color_threshold=0,
                       no_labels=True)
    
    plt.show()
#------------------------------------------------------------------------------




#------------------------------------------------------------------------------
def do_clustering(n, dist_matrix):

    hc = AgglomerativeClustering(n_clusters = n,
                                     #affinity = 'precomputed'
                                     ).fit_predict(dist_matrix)
    labels = hc.labels_
                
    # Todos los elementos son asignados como "-1" ó como "0"
    if len(np.unique(labels)) < 2:
        silh_coef = -1
        cal_hara_coef = 1
        db_coef = 0

    else:
        silh_coef = silhouette_score(dist_matrix,
                                     labels,
                                     metric = 'precomputed')
        cal_hara_coef = calinski_harabasz_score(dist_matrix,
                                                labels)
        db_coef = davies_bouldin_score(dist_matrix,
                                       labels)
                
    return(n, silh_coef, cal_hara_coef, db_coef)
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def do_optimization(dist_matrix, ind, n_clusters):
    
    # Listas que almacenan la combinación de parámetros y los resultados
    indss = list() # Índices de similitud
    nclusts = list()# Valores de número de clusters
    silhouettes = list() # Resultados del coeficiente de silhouette
    calinskis = list() # Resultados del score de calinski-harabasz
    dbs = list() # Resultados del score de davies-bouldin
    
    
    if ind == 'euclidean':
        dist_matrix = np.triu(dist_matrix)
    else:
        dist_matrix = squareform(dist_matrix)
    
    # Recorremos los 4 posibles valores de MinPts
    for n in n_clusters:   
        
        nc, silh_coef, cal_hara_coef, db_coef = do_clustering(n, dist_matrix)
        
        # Guardamos los parámetros y los resultados
        indss.append(ind)
        nclusts.append(nc)
        silhouettes.append(silh_coef)
        calinskis.append(cal_hara_coef)
        dbs.append(db_coef)
    
    results = pd.DataFrame(list(zip(indss,
                                    nclusts,
                                    silhouettes,
                                    calinskis,
                                    dbs)),
                            columns = ['Similarity',
                                       'Clusters',
                                       'Silhouette',
                                       'Calinski-Harabasz',
                                       'Davies-Bouldin'])
    
    return(results)
#------------------------------------------------------------------------------