# -*- coding: utf-8 -*-
"""
Created on Sun Nov 29 12:19:52 2020

@author: Lucia
"""

import pandas as pd
import numpy as np

from sklearn.cluster import KMeans

from sklearn.metrics import pairwise_distances

from sklearn.metrics import silhouette_score
from sklearn.metrics import calinski_harabasz_score
from sklearn.metrics import davies_bouldin_score


#------------------------------------------------------------------------------
def do_clustering(n, dist_matrix): 

    kmeans = KMeans(n_clusters = n).fit(dist_matrix)
                
    labels = kmeans.labels_    
    inertia = kmeans.inertia_ 

    # Todos los elementos son asignados como "-1" ó como "0"
    if len(np.unique(labels)) < 2:
        silh_coef = -1
        cal_hara_coef = 1
        db_coef = 0

    else:
        silh_coef = silhouette_score(dist_matrix,
                                     labels,
                                     metric = 'precomputed')
        cal_hara_coef = calinski_harabasz_score(dist_matrix,
                                                labels)
        db_coef = davies_bouldin_score(dist_matrix,
                                       labels)
                
    return(n, inertia, silh_coef, cal_hara_coef, db_coef)
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def do_optimization(dist_matrix, ind, n_clusters):
    
    # Listas que almacenan la combinación de parámetros y los resultados
    indss = list() # Índices de similitud
    nclusts = list()# Valores de número de clusters
    inertias = list() # Resultados de los valores de dispersion
    silhouettes = list() # Resultados del coeficiente de silhouette
    calinskis = list() # Resultados del score de calinski-harabasz
    dbs = list() # Resultados del score de davies-bouldin
    
    # Recorremos los 4 posibles valores de MinPts
    for n in n_clusters:    

        n, inertia, silh_coef, cal_hara_coef, db_coef = do_clustering(n, dist_matrix)
        
        # Guardamos los parámetros y los resultados
        indss.append(ind)
        nclusts.append(n)
        inertias.append(inertia)
        silhouettes.append(silh_coef)
        calinskis.append(cal_hara_coef)
        dbs.append(db_coef)
    
    results = pd.DataFrame(list(zip(indss,
                                    nclusts,
                                    inertias,
                                    silhouettes,
                                    calinskis,
                                    dbs)),
                            columns = ['Similarity',
                                       'Clusters',
                                       'SSE',
                                       'Silhouette',
                                       'Calinski-Harabasz',
                                       'Davies-Bouldin'])
    
    return(results)
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def simluations(n_simulations, original, index, n_clust, feature, vector, dataset):
    
    original_np = original.to_numpy()
    
    clusters = list()
    inertias = list()
    silhouettes = list()
    calinskis = list() 
    dbs = list() 
    
    for n in range(n_simulations):
        
        shuffled = np.copy(original_np)
    
        # Shuffling elements in each column of the matrix
        for i in range(shuffled.shape[1]):
            np.random.shuffle(shuffled[:,i])
    
        distances = pairwise_distances(shuffled,
                                       metric = index)
    
        n_clusts, inertia, silh_coef, cal_hara_coef, db_coef = do_clustering(n_clust, distances)
        
        clusters.append(n_clusts)
        inertias.append(inertia)
        silhouettes.append(silh_coef)
        calinskis.append(cal_hara_coef)
        dbs.append(db_coef)
    
    results = pd.DataFrame(list(zip(clusters, inertias, silhouettes, calinskis, dbs)),
                          columns = ['Clusters', 'SSE', 'Silhouette', 'Calinski-Harabasz', 'Davies-Bouldin'])
    
    
    results.to_csv("results/simulations/kmeans/" + dataset + '/' + feature + "s_" + vector + '_' + index + "_nclusts" + str(n_clust) + ".csv", index=False)
#------------------------------------------------------------------------------