# -*- coding: utf-8 -*-
"""
Created on Sun Nov 29 11:19:10 2020

@author: Lucia
"""


import numpy as np
from sklearn.metrics import pairwise_distances
import matplotlib.pyplot as plt





#------------------------------------------------------------------------------
def compute(feature_matrix, ind):
    
    """Fuction that from a feature matrix (rows: diseases and columns: features)
    computes the distances matrix (rows: diseases, columns: diseases)
    """
    
    if ind == 'minkowski':
        distances = pairwise_distances(feature_matrix,
                                       metric = ind,
                                       p = 5)
    
    else:
        distances = pairwise_distances(feature_matrix,
                                       metric = ind)
    
    return(distances)
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def visualize_one_plot(matrix, ind, feature, dataset):
    
    """ Function that allows plotting the distributions of distances matrices. 
    """
    
    plt.hist(matrix[np.triu_indices_from(matrix, k=1)], bins=50)
    plt.yscale('log')
    plt.title('Distance distribution: ' + ind + '\nFeature: ' + feature + 's; Dataset: ' + dataset.lower())
    plt.xlabel('Distance value')
    plt.ylabel('Number of times diseases pairs are at that distance (LOGSCALE)')
    plt.show()
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
def visualize_subplots(matrices, features, indexes, dataset, vector):
    
    """ Function that allows plotting the distributions of distances matrices. 
    """
    
    if vector == 'bool':
        fig, axes = plt.subplots(len(features), 
                                 len(indexes), 
                                 figsize=(11,8), 
                                 sharex=True,
                                 sharey=True)
        v = 'binary'
     
    else:
         fig, axes = plt.subplots(len(features), 
                                 len(indexes), 
                                 figsize=(11,4), 
                                 sharey=True)
         v = 'numeric'
         
    
    # for i in range(1, len(features)+1):
    #     for j in range(1, len(indexes)+1):
            
    cols = ['{}'.format(col) for col in indexes]
    rows = ['{}s'.format(row) for row in features]
    
    for ax, col in zip(axes[0], cols):
        ax.set_title(col)
    
    for ax, row in zip(axes[:,0], rows):
        ax.set_ylabel(row, rotation=90, size='large')
    
        
    fig.text(0.5, 0.0, 'Distance value', ha='center')
    fig.text(0.0, 0.5, 'Count in logscale', va='center', rotation='vertical')
    fig.tight_layout()
    
    
    fig.text(0.5, 
             1, 
             'Distance distributions in the case of the ' + dataset.lower() + ' dataset and ' + v + ' vectors', 
             ha='center',
             fontweight = "bold", fontsize=14)
    
    for n in range(len(features)):
        for p in range(len(indexes)):
            axes[n,p].hist(matrices[n][p][np.triu_indices_from(matrices[n][p], k=1)], bins=50)
            
    plt.yscale('log')
    
    plt.show()
#------------------------------------------------------------------------------