# -*- coding: utf-8 -*-
"""
Created on Wed Nov 11 12:03:37 2020

@author: Lucia
"""


# Importing the needed libraries
import time

import numpy as np
import pandas as pd

from sklearn.decomposition import PCA
from sklearn.manifold import TSNE

import matplotlib.pyplot as plt
import colorcet as cc
import seaborn as sns
sns.set_theme(style="whitegrid")

# Variables to set
algorithm = 'optics'
feature = 'gene'
vector = 'real'
metric = 'cosine'
dataset = 'complete'


v = 'numeric'


# Obtaining the data depending on the parameters

feat_matr = pd.read_csv('data/feature_matrices/'+ dataset + '/' + vector +'/'+ feature +'s_feature_matrix.csv')
diseases_genes = pd.read_csv('data/disease_feature/'+ dataset + '/' + vector +'/disease_'+feature+'.csv')
clusters = pd.read_csv('results/clusters/optics_'+dataset+'_'+ vector +'_'+ metric +'_'+ feature +'s.csv')




clusters_count = clusters.groupby('cluster').size().reset_index(name='cluster_items').sort_values('cluster_items')
#sns.barplot(x = 'cluster', y = 'cluster_items', data=clusters_count[(clusters_count.cluster_items>20) & (clusters_count.cluster!=-1)].sort_values('cluster_items'))

clusters = clusters.merge(clusters_count, how = 'inner', on = 'cluster')
clusters.set_index('disease_id', inplace = True)

diseases = diseases_genes['disease_id'].drop_duplicates()
genes = diseases_genes['feature_id'].drop_duplicates()


feat_matr.set_index(diseases, inplace =True)
feat_matr.columns = genes
feat_matr = feat_matr.merge(clusters, how = 'inner', left_index=True, right_index=True)




# =============================================================================
# PCA
# =============================================================================
pca = PCA(n_components=3)
pca_result = pca.fit_transform(feat_matr[genes].values)
feat_matr['pca-one'] = pca_result[:,0]
feat_matr['pca-two'] = pca_result[:,1] 
feat_matr['pca-three'] = pca_result[:,2]
print('Explained variation per principal component: {}'.format(pca.explained_variance_ratio_))


plt.figure(figsize=(16,7))
ax = sns.scatterplot(
    x="pca-one", y="pca-two",
    hue="cluster",
    palette="tab20",
    #data=feat_matr[(feat_matr.cluster_items>20) & (feat_matr.cluster!=-1)],
    data=feat_matr,
    legend=False,
    #alpha=0.3
)
ax.set_title("PCA - " + algorithm + ' - ' + feature + ' - ' + vector + ' - ' + dataset)
plt.show()


ax = plt.figure(figsize=(16,10)).gca(projection='3d')
ax.scatter(
    xs=feat_matr["pca-one"], 
    ys=feat_matr["pca-two"], 
    zs=feat_matr["pca-three"], 
    c=feat_matr["cluster"], 
    cmap='tab10'
)
ax.set_xlabel('pca-one')
ax.set_ylabel('pca-two')
ax.set_zlabel('pca-three')
plt.show()






# =============================================================================
# t-SNE
# =============================================================================
time_start = time.time()
tsne = TSNE(n_components=2, verbose=0, perplexity=40, n_iter=300)
tsne_results = tsne.fit_transform(feat_matr[genes].values)
print('t-SNE done! Time elapsed: {} seconds'.format(time.time()-time_start))

feat_matr['tsne-one'] = tsne_results[:,0]
feat_matr['tsne-two'] = tsne_results[:,1]


plt.figure(figsize=(16,7))
ax = sns.scatterplot(
    x="tsne-one", y="tsne-two",
    hue="cluster",
    palette="tab10",
    data=feat_matr[(feat_matr.cluster_items>20) & (feat_matr.cluster!=-1)],
    #legend=False,
    #alpha=0.3
    size="cluster_items",
    sizes=(10, 100), hue_norm=(0, 7),
)
ax.set_title("tSNE - " + algorithm + ' - ' + feature + ' - ' + vector + ' - ' + dataset)
plt.show()





# =============================================================================
# PCA and t-SNE
# =============================================================================

pca_50 = PCA(n_components=50)
pca_result_50 = pca_50.fit_transform(feat_matr[genes].values)
print('Cumulative explained variation for 50 principal components: {}'.format(np.sum(pca_50.explained_variance_ratio_)))


time_start = time.time()
tsne = TSNE(n_components=2, verbose=0, perplexity=40, n_iter=300)
tsne_pca_results = tsne.fit_transform(pca_result_50)
print('t-SNE done! Time elapsed: {} seconds'.format(time.time()-time_start))

feat_matr['tsne-pca50-one'] = tsne_pca_results[:,0]
feat_matr['tsne-pca50-two'] = tsne_pca_results[:,1]



plt.figure(figsize=(16,7))
ax = sns.scatterplot(
    x="tsne-pca50-one", y="tsne-pca50-two",
    hue="cluster",
    palette='tab10',
    data=feat_matr,
    legend=False,
    #alpha=0.3
)
ax.set_title("PCA + tSNE - " + algorithm + ' - ' + feature + ' - ' + vector + ' - ' + dataset)
plt.show()




plt.figure(figsize=(16,7))
ax1 = plt.subplot(1, 2, 1)
sns.scatterplot(
    x="pca-one", y="pca-two",
    hue="cluster",
    palette='tab10',
    data=feat_matr[(feat_matr.cluster_items>10) & (feat_matr.cluster!=-1)],
    legend=False,
    cmap='tab10',
    ax=ax1
)
ax2 = plt.subplot(1, 2, 2)
sns.scatterplot(
    x="tsne-pca50-one", y="tsne-pca50-two",
    hue="cluster",
    palette='tab10',
    data=feat_matr[(feat_matr.cluster_items>10) & (feat_matr.cluster!=-1)],
    legend=False,
    cmap='tab10',
    ax=ax2
)
plt.title('PCA and PCA+tSNE - '+ algorithm + ' - ' + feature + ' - ' + vector + ' - ' + dataset)
plt.show()

# y = feat_matr[(feat_matr.tsnepca50two > -50) & (feat_matr.cluster!= -1)][['tsne-pca50-one']].to_numpy()
# x = feat_matr[(feat_matr.tsnepca50two > -50) & (feat_matr.cluster!= -1)][['tsnepca50two']].to_numpy()
# jittered_y = y + 0.01 * np.random.rand(len(y)) -0.09
# jittered_x = x + 0.01 * np.random.rand(len(x)) -0.09
# colors = feat_matr[(feat_matr.tsnepca50two > -50)& (feat_matr.cluster!= -1)][['cluster']].to_numpy()

# palette = sns.color_palette(None, feat_matr[(feat_matr.cluster!= -1)& (feat_matr.cluster_items>10)]['cluster'].nunique())




plt.figure(dpi=500)
ax = sns.scatterplot(
# ax = plt.scatter(
    x="tsne-pca50-one", y="tsne-pca50-two",
    hue="cluster",
    palette=cc.cm.glasbey_category10,
    data=feat_matr[(feat_matr.cluster!= -1) & (feat_matr.numbers>10)],
    legend=False,
    size="numbers",
    sizes=(10, 200), 
    alpha = 0.5
    )

plt.title('')
plt.xlabel('1st dimension')
plt.ylabel('2nd dimension')

props = dict(boxstyle='round', facecolor='wheat', alpha=1)
plt.text(-10.25,
        5.75, 
        'Dataset: '+ dataset +'\nVectors: ' + v + '\nDistance: '+metric+'\nFeature: '+feature+'s',
        bbox=props, 
        fontsize = 10)
plt.grid(linewidth = 0.3, linestyle=':')
plt.show()

